#!/usr/bin/env python

__version__ = '$Revision: 4838 $'.split()[1]
__date__ = '$Date: 2006-10-02 10:26:30 -0400 (Mon, 02 Oct 2006) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
Wrap xml option into something nicer and provide associated info for
gengetopt (ggo) and others.

Requires: lxml

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@since: 2006-Sep-26
@status: under development
@organization: U{CCOM<http://ccom.unh.edu/>}


@license: LGPL
'''

import sys

# FIX: when do we get to use bools in c/when is c99 okay to use?!?
cTypes={'bool':'int','int':'int','float':'float','string':'char *'}
'''C language type defintion lookup table'''
cTypesArray={'bool':'int *','int':'int *','float':'float *','string':'char **'}
'''C language type defintion lookup table for arrays of variables'''

cxxTypes={'bool':'bool','int':'int','float':'float','string':'std::string'}
'''C__ language type defintion lookup table'''
# WARNING: vector<bool> is not a proper container!
cxxTypesArray={'bool':'std::vector<bool>','int':'std::vector<int>',
	       'float':'std::vector<float>','string':'std::vector<std::string>'}
'''C++ language type defintion lookup table for arrays of variables.  Arrays are done with vectors.'''

class Option:
    '''
    Provide a nicer interface to the XML element tree for a class::

      <opt name="Input Filename" variable="inputFilename"
	 shortOpt="i" longOpt="input-file" 
	 type="string"
	 default="in.gutm"
	 units="None"
	 repeatable="true"
	 shortDescription="files to read"
	 >Files to read in to the pipeline.
      </opt>

    This class converts units None to the python None object and
    repeatable into a boolean.

    Also adds:
      - cType - for C data type
      - cxxType - for C++ data type
      - gengetoptVar - for the gengetopt structure member on the C processing side
      - longDescription

    '''
    

    def __init__(self,optEt=None,optFile=None,verbose=False):
	'''
	param optEt: elementtree for the option
	param optFile: Use a file or StringIO to present just one
	option.  FIX: implement.  Use xpath to find the first option?
	'''
	self.fields={}
	if verbose: print 'optEt init', optEt
	if optEt!=None: 
	    if verbose: print 'Using etree'
	    assert(optEt.tag=='opt')
	    self.fields['name']=optEt.attrib['name']
	    if verbose: print 'opt name',self.fields['name']
	    self.fields['variable']=optEt.attrib['variable']
	    self.fields['shortOpt']=optEt.attrib['shortOpt']
	    longOpt = self.fields['longOpt']=optEt.attrib['longOpt']
	    varType = self.fields['type']   =optEt.attrib['type']; 

	    self.fields['default']=optEt.attrib['default']
	    if self.fields['default'].lower()=='none':
		self.fields['default']=None
		# FIX: is it right to set to none and let other stages figure out what to do?
		
	    self.fields['units']=optEt.attrib['units']
	    if self.fields['units'].lower()=='none':
		self.fields['units']=None
	    if optEt.attrib['repeatable'].lower()=='true':
		self.fields['repeatable']=True
	    else:
		self.fields['repeatable']=False
	    self.fields['shortDescription']=optEt.attrib['shortDescription']
	    self.fields['longDescription']=optEt.text


	    if self.fields['repeatable']:
		self.fields['cType']=cTypesArray[optEt.attrib['type']]
		self.fields['cxxType']=cxxTypesArray[optEt.attrib['type']]
	    else:
		self.fields['cType']=cTypes[optEt.attrib['type']]
		self.fields['cxxType']=cxxTypes[optEt.attrib['type']]


	    self.fields['gengetoptVar']=longOpt.replace('-','_') 
	    if varType=='bool':
		self.fields['gengetoptVar']+='_flag'
	    else:
		self.fields['gengetoptVar']+='_arg'

	    # AKA length
	    self.fields['gengetoptGiven']=longOpt.replace('-','_')+'_given' 
	    if self.fields['repeatable']:
		self.fields['gengetoptLength']=self.fields['gengetoptGiven']

	else:
	    print 'FIX: set all fields to defaults'
	    assert False

    def __getitem__(self,field):
	'''Allow access to fields with the [] operator.  e.g.
	o = Option(StringIO(optXmlStr))
	print o['name']
	'''
	return self.fields[field]


######################################################################
import unittest

class TestOption(unittest.TestCase):
    '''Test the Option class with a set of option xml definitions'''
    def testStringArray(self):
	'''Validate against an array of strings'''
	from lxml import etree as ET
	boolStr='''<opt name="Input Filename" variable="inputFilename"
	 shortOpt="i" longOpt="input-file" 
	 type="string"
	 default="in.gutm"
	 units="None"
	 repeatable="true"
	 shortDescription="files to read"
	 >Files to read in to the pipeline.</opt>'''
	from StringIO import StringIO
	root = ET.parse(StringIO(boolStr)).getroot()
	opt = Option(optEt=root)

	print 'Name:',opt['name']
	self.failUnless(opt['name']=='Input Filename')
	self.failUnless(opt['variable']=='inputFilename')
	self.failUnless(opt['shortOpt']=='i')
	self.failUnless(opt['longOpt']=='input-file')
	self.failUnless(opt['type']=='string')
	self.failUnless(opt['default']=='in.gutm')
	self.failUnless(opt['units']==None)
	self.failUnless(opt['repeatable'])
	self.failUnless(opt['shortDescription']=='files to read')
	self.failUnless(opt['longDescription']=='Files to read in to the pipeline.')

	self.failUnless(opt['cType']=='char **')
	self.failUnless(opt['cxxType']=='std::vector<std::string>')



if __name__ == '__main__':
    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)
    myparser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
                        help='run the documentation tests')
    myparser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
                        help='run the unit tests')

    (options,args) = myparser.parse_args()

    success=True

    if options.doctest:
	import os; print os.path.basename(sys.argv[0]), 'doctests ...',
	sys.argv= [sys.argv[0]]

	import doctest
	numfail,numtests=doctest.testmod()
	if numfail==0: print 'ok'
	else: 
	    print 'FAILED'
	    success=False

    if not success:
	sys.exit('Something Failed')

    del success # Hide success from epydoc

    if options.unittest:
	sys.argv = [sys.argv[0],'-v']
	unittest.main()
