#!/usr/bin/env python

__version__ = '$Revision: 4838 $'.split()[1]
__date__ = '$Date: 2006-10-02 10:26:30 -0400 (Mon, 02 Oct 2006) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
Read an arg xml definition and produce code for gengetopt ggo file

@requires: U{lxml<http://codespeak.net/lxml/>}
@requires: U{gengetopt<http://www.gnu.org/software/gengetopt/>}

@author: U{'''+__author__+'''<http://xenon.stanford.edu/~schwehr/>}
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@since: 2006-Sep-11
@status: under development
@organization: U{CCOM<http://ccom.unh.edu/>}

@license: LGPL
'''

from genargs.option import Option
import sys

def writeOpt(out,optET):
    '''
    Handle one line of the option file

    @param out: is a file object that will be written to
    @param optET: is an element tree node containing an opt.
    '''
    opt = Option(optET)  # FIX: switch totally to using this Option class object
    o = out

    o.write('option ')
    o.write('"'+optET.attrib['longOpt']+'" ')
    o.write(optET.attrib['shortOpt']+' ')
    o.write('"'+optET.attrib['shortDescription'])
    if None != opt['units']: o.write(' (units: '+opt['units']+')')
    o.write('" ')

    if not opt['repeatable'] and None==opt['default']:
	sys.exit('ERROR: MUST specify defaults for all options.  '+opt['name']+' does not have a default')

    if opt['type']=='bool': 
	o.write('flag ')
	if optET.attrib['default'].lower() == 'false' or opt['default'].lower()==None: o.write('off ')
        else: o.write('on ')
    else: 
	o.write(opt['type']+' ')
	if 'default' in optET.attrib and opt['default']!=None:
	    o.write('default="'+optET.attrib['default']+'" ')
	else:
	    if not opt['repeatable']: print 'WARNING: might want to specify a default for',opt['name']+'!'

    o.write('no ') # Not a required argument... leave this up to the user to program the logic

    if opt['repeatable'] and 'bool' != opt['type']: 
	o.write('multiple')

    if opt['repeatable'] and 'bool' == opt['type']: 
	print '\n\n*** WARNING: gengetopt does not understand arrays of boolean flags!! ***\n'

    o.write('\n')


def build_gengetopts(argsXmlFilename, gengetoptsFile,
		     package='@PACKAGE@',version='@VERSION@',purpose='@PURPOSE@',verbose=True):
    '''
    Load an xml file definiting options and create the gengetopt (ggo)
    file for those options.

    @bug: how do I read from a file so that StringIO is an option?

    @param argXmlFilename: xml file definiting what the options are
    @param gengetoptsFile: file to write out to
    @param package: ggo package field, defaults to easily replaced name for sed or perl
    @param version: ggo version field, defaults to easily replaced name for sed or perl
    @param purpose: ggo purpose field, defaults to easily replaced name for sed or perl
    '''
    from lxml import etree as ET
    root = ET.parse(argsXmlFilename).getroot()

    o = gengetoptsFile
    o.write('''# -*- shell-script -*- Tell emacs this is a shell script for highlighting
# genargs auto-generated command line options
# FIX: specify more about genargs.py: '''+__version__+''' '''+__date__+'''
# http://www.gnu.org/software/gengetopt/gengetopt.html

# Use sed or perl to replace the @ uppercase words of package, version
# and purpose.  e.g.
#
#    perl -pe "s/\@PACKAGE\@/myargs/g" myargs.ggo.in | \\
#    perl -pe "s/\@VERSION\@/0.1/g" | \\
#    perl -pe "s/\@PURPOSE\@/Describe what the package does/g" > myargs.ggo
#
# The version can also be had from a file or other script like this.
# The VERSION file must be only one line that does not have a newline
# at the end.
#
#    perl -pe "s/\@VERSION\@/`cat VERSION`/g"
#
# To build the command line parser do something like this:
#
#    gengetopt --input myargs.ggo --arg-struct-name=myargs_info --file-name=myargs_cmdline



package "'''+package+'''"
version "'''+version+'''"
purpose "'''+purpose+'''"

''')

    if verbose: print 'looping through args'
    assert('args'==root.tag)
    for category in root:
	if not category.tag:
	    continue # skip comments
	for opt in category:
	    if opt.tag!='opt': continue # Skip comments and other stuff
	    #optsDict={}
	    if verbose: print '    opt:', opt.attrib['name']
	    writeOpt(o,opt)
	
######################################################################
if __name__=='__main__':
    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)
    
    myparser.add_option('-g','--gengetopt-file',dest='gengetoptFilename',default='parameters.ggo',
			help='Name of the gengetopt file to write [default: %default]')

    myparser.add_option('-a','--args-xml',dest='argsXmlFilename',default='genargs.xml',
			help='Name of file with the xml arg definitions [default: %default]')

    myparser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
			help='Turn on verbose mode.')


    (options,args) = myparser.parse_args()

    gengetoptsFile = file(options.gengetoptFilename,'w')

    build_gengetopts(options.argsXmlFilename, gengetoptsFile,verbose=options.verbose)

    del gengetoptsFile # prevent epydoc --check from seeing this variable

