#!/usr/bin/env python

__version__ = '$Revision: 4838 $'.split()[1]
__date__ = '$Date: 2006-10-02 10:26:30 -0400 (Mon, 02 Oct 2006) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
Read an arg xml definition and produce python optparse.OptionParser code.

@note: ET is an abreviation for ElementTree.  Generally using the ElementTree API as provided by lxml

@requires: U{epydoc<http://epydoc.sourceforge.net/>}
@requires: U{lxml<http://codespeak.net/lxml/>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser 
@since: 2006-Sep-11
@status: under development
@organization: U{CCOM<http://ccom.unh.edu/>}

@bug: FIX: probably better to make a class from each option, no?
@bug: FIX: Needs a serious cleanup!
@bug: Need to make sure that there are not xml troubles in the encoded text for save
@bug: Should probably use libxml2 for the saving!

@license: LGPL
'''

from genargs.option import Option

######################################################################
def writeOpt(out,optET):
    opt = Option(optET)
    var = opt['variable']
    varType=opt['type']

    out.write('\tparser.add_option(')
    if opt['shortOpt'] and opt['shortOpt']!='-': out.write("'-"+opt['shortOpt']+"',")
    out.write("'--"+opt['longOpt']+"',")
    out.write("dest='"+opt['variable']+"',")

    if opt['default']: out.write("default='"+opt['default']+"',")
    out.write('\n')

    if opt['repeatable']: out.write('\t\t\t action=\'append\',\n')

    if opt['type']=='bool': out.write('\t\t\t action=\'store_true\',\n')

    out.write('\t\t\t help=\''+opt['shortDescription'])
    if opt['default']: out.write(" [default: %default]")
    out.write('\'')
    

    out.write('\n')

    out.write('\t\t\t )\n')
    #out.write('\n')

    return
######################################################################
def writePythonOpt(outFilename='parameters.py'
		   ,xmlFilename='parameters.xml'
		   ,withMain=True
		   ,functionName='parameters'
		   ,verbose=True
		   ):
    '''
    Create a python file that does optparse.OptParse handling

    @param outFilename: python file to write
    @param xmlFile: where to get the config info from
    @param withMain: do we include a main?
    '''
    from lxml import etree as ET
    root = ET.parse(xmlFilename).getroot()

    o = file(outFilename,'w')

    # Put in some funny breaks to prevent svn keyword replacement here

    import datetime
    d = datetime.datetime.utcnow()
    dateStr = str(d.year)+'-'+("%02d" %d.month)+'-'+("%02d"%d.day)

    o.write('''#!/usr/bin/env python

__version__ = '$Rev'''+'''ision: $'.split()[1]
__date__ = '$Da'''+'''te: '''+dateStr+'''$'.split()[1] # utc date
__author__ = 'genpython.py'

__doc__=\'\'\'
optparse.OptionParser code for '''+xmlFilename+'''

Autogenerated code.  DO NOT EDIT.

@note: ET is an abreviation for ElementTree.  Generally using the ElementTree API as provided by lxml

@requires: U{epydoc<http://epydoc.sourceforge.net/>}

@author: \'\'\'+__author__+\'\'\'
@version: \'\'\' + __version__ +\'\'\'
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser 
@status: Autogenerated code.  DO NOT EDIT.
@organization: U{FIX<http://somewhere.com/>}

@license: Unknown
\'\'\'

''')

    o.write('def '+functionName+'(version=None):\n')
    o.write('\t\'\'\'@return an OptionParse object\'\'\'\n')
    o.write('\tfrom optparse import OptionParser\n')
    o.write('\tparser=None\n')
    o.write('\tif None==version: parser = OptionParser(usage="%prog [options]",version="%prog "+__version__)\n')
    o.write('\telse:             parser = OptionParser(usage="%prog [options]",version="%prog "+version)\n')
    o.write('\t\n')

    for category in root:
	if not category.tag:
	    continue # skip comments
	o.write('\n')
	#o.write(indent+'////////////////////////////////////////\n')
	o.write('\t#  Catagory: '+category.attrib['name']+'\n')
	#o.write(indent+'////////////////////////////////////////\n\n')
	for opt in category:
	    if opt.tag!='opt': continue # Skip comments and other stuff
	    if verbose: print '    opt:', opt.attrib['name']
	    writeOpt(o,opt)
	    
    o.write('\n')

    o.write('\treturn parser\n')


    if withMain:
	o.write('''
######################################################################
if __name__=='__main__':
	myparser='''+functionName+'''()
	(options,args) = myparser.parse_args()

	print 'args:\\n\\n',args
	print
	print 'options:\\n\\n',options
''')
	import os
	os.chmod(outFilename,0755)


######################################################################
if __name__=='__main__':
    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)
    
    myparser.add_option('-o','--output',dest='output',default='parameters.py',
			help='Base filename of the gen C++ file to write [default: %default]')

    myparser.add_option('-a','--args-xml',dest='argsXmlFilename',default='genargs.xml',
			help='Name of file with the xml arg definitions [default: %default]')

    myparser.add_option('-m','--with-main',dest='withMain',default=True,action='store_true',
			help='Include a __name__ == __main__ block [default: %default].')
    myparser.add_option('-M','--without-main',dest='withMain',action='store_false',
			help='Do not generate a __name__ == __main__ block.')

    myparser.add_option('-f','--function-name',dest='functionName',default='parameters',
			help='Python function name that returns the parser object [default: %default]')



    (options,args) = myparser.parse_args()


    writePythonOpt(outFilename=options.output
		   ,xmlFilename=options.argsXmlFilename
		   ,withMain=options.withMain
		   ,functionName=options.functionName)


