#!/usr/bin/env python

__version__ = '$Revision: 4838 $'.split()[1]
__date__ = '$Date: 2006-10-02 10:26:30 -0400 (Mon, 02 Oct 2006) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
Create a stub entry for a command line option.  Hopefully this makes
building up xml command line filesmore like cut and paste

@author: U{'''+__author__+'''<http://xenon.stanford.edu/~schwehr/>}
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@since: 2006-Sep-20
@status: under development
@organization: U{CCOM<http://ccom.unh.edu/>}
@license: LGPL
'''

import sys

typeChoices = ['bool','int','float','string']
'''
List of the possible option types.  Any of these can also come in an array flavor.
'''
######################################################################
def build_opt_xml(options):
    '''
    @param options: dictionary of all the fields needed:
    shortDescription, outputFile, complete, default, shortOpt,
    longDescription, repeatable, category, units, longOpt, type
    '''

    #print 'here it goes'
    out = sys.stdout
    if options.outputFile: 
	print 'Switching to output file:',options.outputFile
	out = file(options.outputFile,'w')
    if options.complete: out.write('<?xml version="1.0" encoding="UTF-8"?>\n<args>\n')

    indent='\t'
    out.write(indent+'<category name="'+options.category+'">\n')
    indent+='\t'
    out.write(indent+'<opt name="'+options.name+'" variable="'+options.variable+'"\n')
    indent+='\t'
    if options.shortOpt: out.write(indent+'shortOpt="'+options.shortOpt+'"\n')
    else: out.write(indent+'shortOpt="None"\n') 

    if options.longOpt: out.write(indent+'longOpt="'+options.longOpt+'"\n')
    else: out.write(indent+'longOpt="None"\n') 

    out.write(indent+'whereUsed="'+options.whereUsed+'"\n')
    out.write(indent+'type="'+options.type+'"\n')
    out.write(indent+'default="'+str(options.default)+'"\n')
    out.write(indent+'units="'+str(options.units)+'"\n')

    if options.repeatable: out.write(indent+'repeatable="true"\n')
    else: out.write(indent+'repeatable="false"\n')

    out.write(indent+'shortDescription="'+options.shortDescription+'"\n')
    #out.write(indent+'="'+options.+'"\n')
    out.write(indent+'>\n'+options.longDescription+'\n')
    indent=indent[:-1]
    out.write(indent+'</opt>\n')
    indent=indent[:-1]
    out.write(indent+'</category>\n')

    if options.complete: out.write('</args>\n')

######################################################################
def validate(options,notify=True):
    '''
    Check all of the options to make sure they work
    @param options: dict with all option fields
    @param notify: if true, emit error messages to stdout
    @return: True if options all ok
    '''
    ok=True
    if not options.name: 
	ok=False
	print 'Must specify the "name" of the variable'
    if not options.variable: 
	ok=False
	print 'Must specify "variable" to use'
    if options.shortOpt and len(options.shortOpt) != 1:
	ok=False
	print 'Short opt must only be one character!'

    # FIX: add much more validation?

    return ok

######################################################################
if __name__=='__main__':
    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)
    
    myparser.add_option('-o','--output',dest='outputFile',default=None,
			help='Name of the xml file to write [default: stdout]')


    myparser.add_option('-n','--name',dest='name',default=None,
			help='Name of the option.  Make this like "Input Filename"')

    myparser.add_option('-v','--variable',dest='variable',default=None,
			help='Name of the variable to write (e.g. inputFilename)')


    myparser.add_option('-C','--complete',dest='complete',default=False, action='store_true',
			help='Write a complete xml file? [default: No])') #%default]')
    
    myparser.add_option('-c','--category',dest='category',default='default',
			help='Which category to drop the option into [default: %default]')

    myparser.add_option('-s','--short',dest='shortOpt',default=None,
			help='Short command line option [default: %default]')

    myparser.add_option('-l','--long',dest='longOpt',default=None,
			help='Long command line option.  REQUIRED [default: %default]')

    myparser.add_option('-w','--where-used',dest='whereUsed',default='',
			help='Space separated list of classes where this option is used.'
			+'e.g. "Defocus Generalize TestAlgorithm12"')

    myparser.add_option('-d','--default',dest='default',default=None,type='string',
			help='Default [default: %default]')

    myparser.add_option('-t','--type',dest='type', type='choice', default='bool',
			choices=typeChoices,
			help='What type of data will the option represent? ('+
			(''.join([choice +' ' for choice in typeChoices]))[:-1]+')'
			+' [default: %default]')

    myparser.add_option('-u','--units',dest='units',default=None,
			help='Units (e.g. m, km, C, T, kg) [default: %default]')

    myparser.add_option('-r','--repeatable',dest='repeatable',default=False, action='store_true',
			help='If the option can be specified multiple times (aka an array)'+
			' [default: %default]'
			)

    myparser.add_option('-S','--short-description',dest='shortDescription',type='string',
			default='FIX: add short description here',
			help='Short description text for this option.  Make it less than 60 characters.'
			)
    myparser.add_option('-L','--long-description',dest='longDescription',type='string',
			default='FIX: add multi paragraph description here',
			help='Generally, it is better to just edit this in the resulting file')

    (options,args) = myparser.parse_args()

    print options

    if not validate(options):
	sys.exit('\nERROR: Fix your options and try again.  Use -h for help')


    build_opt_xml(options)
